///File : TextBase.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "TextBase.hpp"
//------------------------------------------------------------------------------

namespace mdo
{

//------------------------------------------------------------------------------
//---------------------------------- Text --------------------------------------
//------------------------------------------------------------------------------
bool TextBase::CalculSizeReal(wxDC *dc, const int &width, const int &askedHeight)
{
    // Font
    DCSelectColorFont(dc, m_data.font);

    m_sizeReal.SetWidth(width);
    int maxTextWidth = m_sizeReal.GetWidth() - m_cfg->GetInt("tableVlineMarging") * 2;

    // Arrange Text
    m_data.textToDraw = ArrangeTextForWidth(dc, m_data.textToDraw, maxTextWidth);

    // Calcule de la hauteur...
    int textHeight = GetTextSize(dc, m_data.textToDraw).GetHeight();
    textHeight += m_cfg->GetInt("tableHlineMarging") * 2;

    // Est-elle ok avec la hauteur demandée ?
    bool heightOk;
    if(textHeight <= askedHeight) // Ça rentre dans plus petit : on agrandi la zone
    {
        m_sizeReal.SetHeight(askedHeight);
        heightOk = true;
    }
    else
    {
        m_sizeReal.SetHeight(textHeight);
        heightOk = false;
    }

    // Ok
    m_isReadyToDraw = true;

    return heightOk;
}
//------------------------------------------------------------------------------
TextBase::TextBase(ConfigManager *cfg, ConfigManager *devisCfg, DrawableObject *parrentSizer, const std::string &name) : DrawableObject(cfg, devisCfg, parrentSizer, name)
{

}
//------------------------------------------------------------------------------
void TextBase::LoadFromCfg(const std::string &name)
{
    DrawableObject::LoadFromCfg(name);
    m_data.isBorder = m_cfg->GetBool(m_cfgPrefix + "isBorder");
    m_data.font = stdColorFontToWxColorFont(m_cfg->GetColorFont(m_cfgPrefix + "font"));

    m_data.vAlign = m_cfg->GetInt(m_cfgPrefix + "vAlign");
    m_data.hAlign = m_cfg->GetInt(m_cfgPrefix + "hAlign");
}
//------------------------------------------------------------------------------
void TextBase::SaveToCfg(const std::string &name)
{
    DrawableObject::SaveToCfg(name);
    m_cfg->SetBool(m_cfgPrefix + "isBorder", m_data.isBorder);
    m_cfg->SetColorFont(m_cfgPrefix + "font", wxColorFontToStdColorFont(m_data.font));

    m_cfg->SetInt(m_cfgPrefix + "vAlign", m_data.vAlign);
    m_cfg->SetInt(m_cfgPrefix + "hAlign", m_data.hAlign);
}
//------------------------------------------------------------------------------
void TextBase::CreateToCfg(const std::string &name)
{
    DrawableObject::CreateToCfg(name);
    m_cfg->SetBool(m_cfgPrefix + "isBorder", false);

    StructStdColorFont colorFont;
    colorFont.font = wxFontToStdFont(wxSystemSettings::GetFont(wxSYS_SYSTEM_FONT ));
    colorFont.font.pointSize = 48;
    colorFont.color.r = 0;
    colorFont.color.g = 0;
    colorFont.color.b = 0;

    m_cfg->SetColorFont(m_cfgPrefix + "font", colorFont);

    m_cfg->SetInt(m_cfgPrefix + "vAlign", vAlign_Left);
    m_cfg->SetInt(m_cfgPrefix + "hAlign", hAlign_Top);
}
//------------------------------------------------------------------------------
void TextBase::RemoveFromCfg(const std::string &name, const bool &withChilds)
{
    DrawableObject::RemoveFromCfg(name);
    m_cfg->RemoveBool(m_cfgPrefix + "isBorder");
    m_cfg->RemoveColorFont(m_cfgPrefix + "font");

    m_cfg->RemoveInt(m_cfgPrefix + "vAlign");
    m_cfg->RemoveInt(m_cfgPrefix + "hAlign");
}
//------------------------------------------------------------------------------
void TextBase::EditPropertyDialog(DialogProperty *window)
{
    DrawableObject::EditPropertyDialog(window);

    wxPanel *PagePanel;
    wxBoxSizer *PageSizer;

    // Special Mdo
    PagePanel = new wxPanel(window->GetNotebook(), wxID_ANY);
    PageSizer = new wxBoxSizer(wxVERTICAL);

    window->AddParamCtrl(type_check, PagePanel, PageSizer, m_cfgPrefix + "isBorder", _("Add border"));

    PageSizer->AddSpacer(20);

    wxArrayString wxVAlignNames(vAlign_nb, vAlign_Names);
    window->AddParamCtrl(type_choice_index, PagePanel, PageSizer, m_cfgPrefix + "vAlign", _("Vertical align :"), static_cast<void*>(&wxVAlignNames));

    wxArrayString wxHAlignNames(hAlign_nb, hAlign_Names);
    window->AddParamCtrl(type_choice_index, PagePanel, PageSizer, m_cfgPrefix + "hAlign", _("Horizontal align :"), static_cast<void*>(&wxHAlignNames));

    PageSizer->AddSpacer(20);

    window->AddParamCtrl(type_color_font, PagePanel, PageSizer, m_cfgPrefix + "font", _("Font"));

    PagePanel->SetSizer(PageSizer);
    window->GetNotebook()->AddPage(PagePanel, _("Object"));
}
//------------------------------------------------------------------------------
void TextBase::Draw(wxDC *dc, const wxPoint &pos)
{
    ThrowIfNotReadyToDraw();

    // Pos
    wxPoint drawPos;
    switch(m_data.vAlign)
    {
        case vAlign_Left:
            drawPos.x = pos.x + m_cfg->GetInt("tableVlineMarging");
            break;
        case vAlign_Center:
            drawPos.x = pos.x + GetSizeReal().GetWidth() / 2;
            break;
        case vAlign_Right:
            drawPos.x = pos.x + GetSizeReal().GetWidth() - m_cfg->GetInt("tableVlineMarging");
            break;
        default:
            std::string msg = "Unknown valign " + nbToString(m_data.vAlign);
            throw ticpp::Exception(msg.c_str());
            break;
    }

    switch(m_data.hAlign)
    {
        case hAlign_Top:
            drawPos.y = pos.y + m_cfg->GetInt("tableHlineMarging");
            break;
        case hAlign_Center:
            drawPos.y = pos.y + GetSizeReal().GetHeight() / 2;
            break;
        case hAlign_Bottom:
            drawPos.y = pos.y + GetSizeReal().GetHeight() - m_cfg->GetInt("tableHlineMarging");
            break;
        default:
            std::string msg = "Unknown halign " + nbToString(m_data.hAlign);
            throw ticpp::Exception(msg.c_str());
            break;
    }

    // Draw
    DCSelectColorFont(dc, m_data.font);
    DrawText(dc, drawPos, m_data.textToDraw, m_data.vAlign, m_data.hAlign);

    if(m_data.isBorder)
    {
        // Bordures
        dc->DrawRoundedRectangle(pos.x, pos.y, GetSizeReal().GetWidth(), GetSizeReal().GetHeight(), m_cfg->GetInt("arcSize"));
    }
}
//------------------------------------------------------------------------------


} // namespace mdo
